<?php

/***************************************************************************
 *
 * Copyright (c) 2020 ksyun.com, Inc. All Rights Reserved
 *
 **************************************************************************/

require_once(dirname(__FILE__) . '/KscUtils.php');

/**
 * Client for ksyun onepass service.
 * 
 * @package ksyun
 * @version v1.0.0
 */
class KOnePassApi
{	

	const SERVICE_NAME    = 'onepass';
	const SERVICE_VERSION = '2019-05-01';
    const SERVICE_DOMAIN  = 'https://onepass.api.ksyun.com';

    protected $accessKey;
    protected $secretKey;
    
    /**
     * KOnePassApi constructor.
     * @param $accessKey
     * @param $secretKey
     */
    public function __construct($accessKey, $secretKey)
    {
    	$this->accessKey = $accessKey;
    	$this->secretKey = $secretKey;
	}
	
    /**
     * Notes: 构造公共参数
     * Date: 2020-07-22 15:16
     * @param $action
     * @return array
     */
	public function buildSysParams($action)
    {
        $sys_params = [
			'Accesskey' => $this->accessKey,
			'Service'	=> self::SERVICE_NAME,
			'Action'    => $action,
			'Version'	=> self::SERVICE_VERSION,
			'Timestamp' => gmdate('Y-m-d\TH:i:s\Z'),
			'SignatureVersion' => '1.0',
			'SignatureMethod'  => 'HMAC-SHA256',
		];
		return $sys_params;
    }
	
    /**
     * Notes:
     * 一键登录
     * https://docs.ksyun.com/documents/37018
     *
     * Date: 2020-07-22 14:42
     * @param $appId      应用的APPID
     * @param $processId  端上sdk中返回的 process_id
     * @param $token      端上sdk中返回的 token
     * @param string $authCode 端上sdk中返回的 运营商校验码 电信有值、移动联通传空字符串 sdk为安卓 0.8.0/IOS 1.8.0及以上版本时，服务端均需传此参数
     * @return bool|false|mixed|string
     */
	public function mobileQuery($appId, $processId, $token, $authCode = '')
	{
		$sysParams  = self::buildSysParams('MobileQuery');
		$params     = array(
			'AppId'     => $appId,
			'Token'     => $processId . ' ' . $token,
            'AuthCode'  => $authCode,
		);

		$params = array_merge($sysParams, $params);
		return $this->makeRequest($params);
	}

    /**
     * Notes:
     * 本机认证
     * https://docs.ksyun.com/documents/37080
     *
     * Date: 2020-07-22 14:49
     * @param $appId      应用的APPID
     * @param $processId  端上sdk中返回的 process_id
     * @param $token      端上sdk中返回的 token
     * @param $mobile     待校验的手机号码
     * @return bool|false|mixed|string
     */
	public function mobileValidate($appId, $processId, $token, $mobile)
	{
		$sysParams  = self::buildSysParams('MobileValidate');
		$params     = array(
			'AppId'	    => $appId,
			'Token'     => $processId . ' ' . $token,
            'Mobile'	=> $mobile,
		);

		$params = array_merge($sysParams, $params);
		return $this->makeRequest($params);
	}

    /**
     * Notes:
     * 本机认证 (H5 小程序）
     * https://docs.ksyun.com/documents/37549
     *
     * Date: 2020-07-22 15:13
     * @param $appId      应用的APPID
     * @param $processId  端上sdk中返回的 process_id
     * @param $token      端上sdk中返回的 token
     * @param $mobile     待校验的手机号码
     * @return bool|false|mixed|string
     */
	public function mobileWebValidate($appId, $processId, $token, $mobile)
	{
		$sysParams = self::buildSysParams('MobileWebValidate');
        $params = array(
            'AppId'	    => $appId,
            'Token'     => $processId . ' ' . $token,
            'Mobile'	=> $mobile,
        );

		$params = array_merge($sysParams, $params);
		return $this->makeRequest($params);
	}

    /**
     * Notes: 发送请求
     * Date: 2020-07-22 15:16
     * @param $params
     * @param string $http_method
     * @return bool|false|mixed|string
     */
	public function makeRequest($params, $http_method = 'POST')
	{
		$params['Signature'] = KscUtils::generateSign($params, $this->secretKey);
		$result = KscUtils::request(self::SERVICE_DOMAIN, $params, $http_method);
		if ($result) {
			$result = json_decode($result['http_body'], true);
			return $result;
		}
		
		return false;
	}
}

 
/* vim: set expandtab ts=4 sw=4 sts=4 tw=100: */