<?php

/***************************************************************************
 *
 * Copyright (c) 2019 ksyun.com, Inc. All Rights Reserved
 *
 **************************************************************************/


/**
 * Utils class for ksyun OpenAPI SDK.
 * 
 * @package ksyun
 * @version v1.0.0
 */
class KscUtils
{

	private static $errno = 0;
    private static $errmsg = '';
    private static $isDebug = false;
    
    /**
     * Set the gloable error number and error message.
     * 
     * @param int $errno Error code
     * @param string $errmsg Error message
     * @return void
     */
    public static function setError($errno, $errmsg)
    {
    	self::$errno = $errno;
        self::$errmsg = $errmsg;
        $msg = sprintf('errcode[%s] errmsg[%s]', $errno, $errmsg);
    	self::errorLog($msg);
    }
    
    /**
     * Get the gloable errno.
     * 
     * @return int
     */
    public static function errno()
    {
    	return self::$errno;
    }
    
    /**
     * Get the gloable error message.
     * 
     * @return string
     */
    public static function errmsg()
    {
    	return self::$errmsg;
    }
    
    /**
     * Whether to set the debug mode of the ksc OpenAPI SDK or not.
     * 
     * @param bool $on true or false
     * @return void
     */
    public static function setDebugMode($on = true)
    {
    	self::$isDebug = $on;
    }
    
    /**
     * Whether the debug mode of the ksc OpenAPI SDK is on or off.
     * 
     * @return bool
     */
    public static function isDebugMode()
    {
    	return self::$isDebug;
    }
    
	/**
     * Request for a http/https resource
     * 
     * @param string $url Url to request
     * @param array $params Parameters for the request
     * @param string $httpMethod Http method, 'GET' or 'POST'
     * @return string|false Returns string if success, or false if failed
     */
	public static function request($url, $params = array(), $httpMethod = 'GET', $connect_timeout = 3, $timeout = 5)
    {

    	$ch = curl_init();
  		
    	$curl_opts = array(
			CURLOPT_CONNECTTIMEOUT	=> $connect_timeout,
			CURLOPT_TIMEOUT			=> $timeout,
			CURLOPT_USERAGENT		=> 'ksc-apiclient-php-1.0',
	    	CURLOPT_HTTP_VERSION	=> CURL_HTTP_VERSION_1_1,
	    	CURLOPT_RETURNTRANSFER	=> true,
	    	CURLOPT_HEADER			=> false,
            CURLOPT_FOLLOWLOCATION	=> false,
            CURLOPT_HTTPHEADER => array(
                "accept: application/json",
              ),
		);

		if (stripos($url, 'https://') === 0) {
			$curl_opts[CURLOPT_SSL_VERIFYPEER] = false;
		}
		
		if (strtoupper($httpMethod) === 'GET') {
			$query = http_build_query($params, '', '&');
			$delimiter = strpos($url, '?') === false ? '?' : '&';
    		$curl_opts[CURLOPT_URL] = $url . $delimiter . $query;
    		$curl_opts[CURLOPT_POST] = false;
		} else {
            $body = http_build_query($params, '', '&');
			$curl_opts[CURLOPT_URL] = $url;
    		$curl_opts[CURLOPT_POSTFIELDS] = $body;
		}
    
    	curl_setopt_array($ch, $curl_opts);
        $result    = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

    	if ($result === false) {
    		self::setError(curl_errno($ch), curl_error($ch));
            curl_close($ch);
            return false;
    	} elseif (empty($result)) {
    		if ($http_code != 200) {
    			self::setError($http_code, 'http response status code: ' . $http_code);
    			curl_close($ch);
    			return false;
    		}
    	}
        
    	curl_close($ch);
    	
    	return ['http_body' => $result, 'http_code' => $http_code];
    }
    
	/**
	 * Prints to the error log
	 *
	 * @param String log message
	 */
	public static function errorLog($msg)
	{	
		// Set the debug mode if you want to see the errors on the page
		if (self::$isDebug) {
			echo 'error_log: '.$msg."\n";
		}
	}
	
	/**
	 * Generate the signature for passed parameters.
	 * 
	 * @param array $params Array of parameters to be signatured
	 * @param string $secret_key Secret key for signature
	 * @param string $namespace The parameter which will be excluded when calculate the signature
	 * @return string Signature of the parameters
	 */
	public static function generateSign($params, $secret_key, $namespace = 'Signature')
    {
        ksort($params, SORT_STRING);
        $str_encode = '';
        foreach($params as $k=>$v) {
            if ($k == $namespace) {
                continue;
            }
            $str_encode .= rawurlencode($k).'='.rawurlencode($v).'&';
        }
        $str_encode = substr($str_encode, 0, -1);
        return hash_hmac("sha256", $str_encode, $secret_key);
    }

}

 
/* vim: set expandtab ts=4 sw=4 sts=4 tw=100: */